@@ -42,10 +42,9 @@
  * #ClutterCanvas is available since Clutter 1.10.
  */
 
-#ifdef HAVE_CONFIG_H
 #include "clutter-build-config.h"
-#endif
 
+#include <math.h>
 #include <cogl/cogl.h>
 #include <cairo-gobject.h>
 
@@ -71,14 +70,12 @@ struct _ClutterCanvasPrivate
 
   int width;
   int height;
+  float scale_factor;
 
   CoglTexture *texture;
   gboolean dirty;
 
   CoglBitmap *buffer;
-
-  int scale_factor;
-  guint scale_factor_set : 1;
 };
 
 enum
@@ -88,7 +85,6 @@ enum
   PROP_WIDTH,
   PROP_HEIGHT,
   PROP_SCALE_FACTOR,
-  PROP_SCALE_FACTOR_SET,
 
   LAST_PROP
 };
@@ -104,7 +100,7 @@ enum
 
 static guint canvas_signals[LAST_SIGNAL] = { 0, };
 
-static void clutter_content_iface_init (ClutterContentIface *iface);
+static void clutter_content_iface_init (ClutterContentInterface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (ClutterCanvas, clutter_canvas, G_TYPE_OBJECT,
                          G_ADD_PRIVATE (ClutterCanvas)
@@ -186,8 +182,16 @@ clutter_canvas_set_property (GObject      *gobject,
       break;
 
     case PROP_SCALE_FACTOR:
-      clutter_canvas_set_scale_factor (CLUTTER_CANVAS (gobject),
-                                       g_value_get_int (value));
+      {
+        gfloat new_scale_factor = g_value_get_float (value);
+
+        if (priv->scale_factor != new_scale_factor)
+          {
+            priv->scale_factor = new_scale_factor;
+
+            clutter_content_invalidate (CLUTTER_CONTENT (gobject));
+          }
+      }
       break;
 
     default:
@@ -215,14 +219,7 @@ clutter_canvas_get_property (GObject    *gobject,
       break;
 
     case PROP_SCALE_FACTOR:
-      if (priv->scale_factor_set)
-        g_value_set_int (value, priv->scale_factor);
-      else
-        g_value_set_int (value, -1);
-      break;
-
-    case PROP_SCALE_FACTOR_SET:
-      g_value_set_boolean (value, priv->scale_factor_set);
+      g_value_set_float (value, priv->scale_factor);
       break;
 
     default:
@@ -268,46 +265,19 @@ clutter_canvas_class_init (ClutterCanvasClass *klass)
                       G_PARAM_READWRITE |
                       G_PARAM_STATIC_STRINGS);
 
-  /**
-   * ClutterCanvas:scale-factor-set:
-   *
-   * Whether the #ClutterCanvas:scale-factor property is set.
-   *
-   * If the #ClutterCanvas:scale-factor-set property is %FALSE
-   * then #ClutterCanvas will use the #ClutterSettings:window-scaling-factor
-   * property.
-   *
-   * Since: 1.18
-   */
-  obj_props[PROP_SCALE_FACTOR_SET] =
-    g_param_spec_boolean ("scale-factor-set",
-                          P_("Scale Factor Set"),
-                          P_("Whether the scale-factor property is set"),
-                          FALSE,
-                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
-
   /**
    * ClutterCanvas:scale-factor:
    *
-   * The scaling factor to be applied to the Cairo surface used for
-   * drawing.
-   *
-   * If #ClutterCanvas:scale-factor is set to a negative value, the
-   * value of the #ClutterSettings:window-scaling-factor property is
-   * used instead.
-   *
-   * Use #ClutterCanvas:scale-factor-set to check if the scale factor
-   * is set.
-   *
-   * Since: 1.18
+   * The height of the canvas.
    */
   obj_props[PROP_SCALE_FACTOR] =
-    g_param_spec_int ("scale-factor",
-                      P_("Scale Factor"),
-                      P_("The scaling factor for the surface"),
-                      -1, 1000,
-                      -1,
-                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+    g_param_spec_float ("scale-factor",
+                        P_("Scale Factor"),
+                        P_("The Scale factor of the canvas"),
+                        0.01f, G_MAXFLOAT,
+                        1.0f,
+                        G_PARAM_READWRITE |
+                        G_PARAM_STATIC_STRINGS);
 
   /**
    * ClutterCanvas::draw:
@@ -354,13 +324,14 @@ clutter_canvas_init (ClutterCanvas *self)
 
   self->priv->width = -1;
   self->priv->height = -1;
-  self->priv->scale_factor = -1;
+  self->priv->scale_factor = 1.0f;
 }
 
 static void
-clutter_canvas_paint_content (ClutterContent   *content,
-                              ClutterActor     *actor,
-                              ClutterPaintNode *root)
+clutter_canvas_paint_content (ClutterContent      *content,
+                              ClutterActor        *actor,
+                              ClutterPaintNode    *root,
+                              ClutterPaintContext *paint_context)
 {
   ClutterCanvas *self = CLUTTER_CANVAS (content);
   ClutterCanvasPrivate *priv = self->priv;
@@ -397,28 +368,18 @@ clutter_canvas_emit_draw (ClutterCanvas *self)
   gboolean mapped_buffer;
   unsigned char *data;
   CoglBuffer *buffer;
-  int window_scale = 1;
   gboolean res;
   cairo_t *cr;
 
-  g_assert (priv->width > 0 && priv->width > 0);
+  g_assert (priv->height > 0 && priv->width > 0);
 
   priv->dirty = TRUE;
 
-  if (priv->scale_factor_set)
-    window_scale = priv->scale_factor;
-  else
-    g_object_get (clutter_settings_get_default (),
-                  "window-scaling-factor", &window_scale,
-                  NULL);
+  real_width = ceilf (priv->width * priv->scale_factor);
+  real_height = ceilf (priv->height * priv->scale_factor);
 
-  real_width = priv->width * window_scale;
-  real_height = priv->height * window_scale;
-
-  CLUTTER_NOTE (MISC, "Creating Cairo surface with size %d x %d (real: %d x %d, scale: %d)",
-                priv->width, priv->height,
-                real_width, real_height,
-                window_scale);
+  CLUTTER_NOTE (MISC, "Creating Cairo surface with size %d x %d",
+                priv->width, priv->height);
 
   if (priv->buffer == NULL)
     {
@@ -461,7 +422,9 @@ clutter_canvas_emit_draw (ClutterCanvas *self)
       mapped_buffer = FALSE;
     }
 
-  cairo_surface_set_device_scale (surface, window_scale, window_scale);
+  cairo_surface_set_device_scale (surface,
+                                  priv->scale_factor,
+                                  priv->scale_factor);
 
   self->priv->cr = cr = cairo_create (surface);
 
@@ -524,16 +487,16 @@ clutter_canvas_get_preferred_size (ClutterContent *content,
     return FALSE;
 
   if (width != NULL)
-    *width = priv->width;
+    *width = ceilf (priv->width * priv->scale_factor);
 
   if (height != NULL)
-    *height = priv->height;
+    *height = ceilf (priv->height * priv->scale_factor);
 
   return TRUE;
 }
 
 static void
-clutter_content_iface_init (ClutterContentIface *iface)
+clutter_content_iface_init (ClutterContentInterface *iface)
 {
   iface->invalidate = clutter_canvas_invalidate;
   iface->paint_content = clutter_canvas_paint_content;
@@ -640,77 +603,44 @@ clutter_canvas_set_size (ClutterCanvas *canvas,
 /**
  * clutter_canvas_set_scale_factor:
  * @canvas: a #ClutterCanvas
- * @scale: the scale factor, or -1 for the default
+ * @scale: the integer scaling factor of the canvas
  *
- * Sets the scaling factor for the Cairo surface used by @canvas.
+ * Sets the scaling factor of the @canvas, and invalidates the content.
  *
- * This function should rarely be used.
- *
- * The default scaling factor of a #ClutterCanvas content uses the
- * #ClutterSettings:window-scaling-factor property, which is set by
- * the windowing system. By using this function it is possible to
- * override that setting.
- *
- * Changing the scale factor will invalidate the @canvas.
- *
- * Since: 1.18
+ * This function will cause the @canvas to be invalidated only
+ * if the scale factor of the canvas surface has changed.
  */
 void
 clutter_canvas_set_scale_factor (ClutterCanvas *canvas,
-                                 int            scale)
+                                 float          scale)
 {
-  ClutterCanvasPrivate *priv;
-  GObject *obj;
-
   g_return_if_fail (CLUTTER_IS_CANVAS (canvas));
-  g_return_if_fail (scale != 0);
+  g_return_if_fail (scale > 0.0f);
 
-  priv = canvas->priv;
-
-  if (scale < 0)
+  if (canvas->priv->scale_factor != scale)
     {
-      if (!priv->scale_factor_set)
-        return;
+      canvas->priv->scale_factor = scale;
 
-      priv->scale_factor_set = FALSE;
-      priv->scale_factor = -1;
+      g_object_freeze_notify (G_OBJECT (canvas));
+      clutter_content_invalidate (CLUTTER_CONTENT (canvas));
+      g_object_thaw_notify (G_OBJECT (canvas));
+
+      g_object_notify_by_pspec (G_OBJECT (canvas), obj_props[PROP_SCALE_FACTOR]);
     }
-  else
-    {
-      if (priv->scale_factor_set && priv->scale_factor == scale)
-        return;
-
-      priv->scale_factor_set = TRUE;
-      priv->scale_factor = scale;
-    }
-
-  clutter_content_invalidate (CLUTTER_CONTENT (canvas));
-
-  obj = G_OBJECT (canvas);
-
-  g_object_notify_by_pspec (obj, obj_props[PROP_SCALE_FACTOR]);
-  g_object_notify_by_pspec (obj, obj_props[PROP_SCALE_FACTOR_SET]);
 }
 
 /**
  * clutter_canvas_get_scale_factor:
  * @canvas: a #ClutterCanvas
  *
- * Retrieves the scaling factor of @canvas, as set using
- * clutter_canvas_set_scale_factor().
+ * Gets the scale factor of the @canvas.
  *
- * Return value: the scaling factor, or -1 if the @canvas
- *   uses the default from #ClutterSettings
- *
- * Since: 1.18
+ * Return value: the current @canvas scale factor or -1 if invalid
  */
-int
+float
 clutter_canvas_get_scale_factor (ClutterCanvas *canvas)
 {
-  g_return_val_if_fail (CLUTTER_IS_CANVAS (canvas), -1);
-
-  if (!canvas->priv->scale_factor_set)
-    return -1;
+  g_return_val_if_fail (CLUTTER_IS_CANVAS (canvas), -1.0f);
 
   return canvas->priv->scale_factor;
 }
