@@ -55,9 +55,9 @@
 #include "clutter-backend-private.h"
 #include "clutter-config.h"
 #include "clutter-debug.h"
-#include "clutter-device-manager-private.h"
 #include "clutter-event-private.h"
 #include "clutter-feature.h"
+#include "clutter-input-device-private.h"
 #include "clutter-input-pointer-a11y-private.h"
 #include "clutter-graphene.h"
 #include "clutter-main.h"
@@ -1523,7 +1523,7 @@ event_click_count_generate (ClutterEvent *event)
         previous_y = event->button.y;
         previous_time = event->button.time;
 
-        /* fallthrough */
+        G_GNUC_FALLTHROUGH;
       case CLUTTER_BUTTON_RELEASE:
         event->button.click_count = click_count;
         break;
@@ -1776,7 +1776,7 @@ _clutter_process_event_details (ClutterActor        *stage,
 
             emit_crossing_event (event, device);
 
-            actor = _clutter_input_device_update (device, NULL, FALSE);
+            actor = clutter_input_device_update (device, NULL, FALSE);
             if (actor != stage)
               {
                 ClutterEvent *crossing;
@@ -1834,13 +1834,10 @@ _clutter_process_event_details (ClutterActor        *stage,
           {
             if (_clutter_is_input_pointer_a11y_enabled (device))
               {
-                ClutterInputDevice *core_pointer;
                 gfloat x, y;
 
                 clutter_event_get_coords (event, &x, &y);
-                core_pointer = clutter_device_manager_get_core_device (device->device_manager,
-                                                                       CLUTTER_POINTER_DEVICE);
-                _clutter_input_pointer_a11y_on_motion_event (core_pointer, x, y);
+                _clutter_input_pointer_a11y_on_motion_event (device, x, y);
               }
           }
 #endif /* CLUTTER_WINDOWING_X11 */
@@ -1871,7 +1868,7 @@ _clutter_process_event_details (ClutterActor        *stage,
             break;
           }
 
-      /* fallthrough from motion */
+        G_GNUC_FALLTHROUGH;
       case CLUTTER_BUTTON_PRESS:
       case CLUTTER_BUTTON_RELEASE:
 #ifdef CLUTTER_WINDOWING_X11
@@ -1879,12 +1876,7 @@ _clutter_process_event_details (ClutterActor        *stage,
           {
             if (_clutter_is_input_pointer_a11y_enabled (device) && (event->type != CLUTTER_MOTION))
               {
-                ClutterInputDevice *core_pointer;
-
-                core_pointer = clutter_device_manager_get_core_device (device->device_manager,
-                                                                       CLUTTER_POINTER_DEVICE);
-
-                _clutter_input_pointer_a11y_on_button_event (core_pointer,
+                _clutter_input_pointer_a11y_on_button_event (device,
                                                              event->button.button,
                                                              event->type == CLUTTER_BUTTON_PRESS);
               }
@@ -1943,7 +1935,7 @@ _clutter_process_event_details (ClutterActor        *stage,
                * get the actor underneath
                */
               if (device != NULL)
-                actor = _clutter_input_device_update (device, NULL, TRUE);
+                actor = clutter_input_device_update (device, NULL, TRUE);
               else
                 {
                   CLUTTER_NOTE (EVENT, "No device found: picking");
@@ -2016,7 +2008,7 @@ _clutter_process_event_details (ClutterActor        *stage,
             break;
           }
 
-      /* fallthrough from motion */
+        G_GNUC_FALLTHROUGH;
       case CLUTTER_TOUCH_BEGIN:
       case CLUTTER_TOUCH_CANCEL:
       case CLUTTER_TOUCH_END:
@@ -2060,7 +2052,7 @@ _clutter_process_event_details (ClutterActor        *stage,
                 }
 
               if (device != NULL)
-                actor = _clutter_input_device_update (device, sequence, TRUE);
+                actor = clutter_input_device_update (device, sequence, TRUE);
               else
                 {
                   CLUTTER_NOTE (EVENT, "No device found: picking");
