@@ -124,6 +124,7 @@ add_stencil_clip_region (CoglFramebuffer *framebuffer,
   CoglMatrix matrix;
   int num_rectangles = cairo_region_num_rectangles (region);
   int i;
+  CoglVertexP2 *vertices;
 
   /* NB: This can be called while flushing the journal so we need
    * to be very conservative with what state we change.
@@ -167,31 +168,50 @@ add_stencil_clip_region (CoglFramebuffer *framebuffer,
       GE( ctx, glStencilOp (GL_KEEP, GL_KEEP, GL_REPLACE) );
     }
 
+  vertices = g_alloca (sizeof (CoglVertexP2) * num_rectangles * 6);
+
   for (i = 0; i < num_rectangles; i++)
     {
       cairo_rectangle_int_t rect;
-      float tl[4], br[4];
+      float x1, y1, z1, w1;
+      float x2, y2, z2, w2;
+      CoglVertexP2 *v = vertices + i * 6;
 
       cairo_region_get_rectangle (region, i, &rect);
 
-      tl[0] = rect.x;
-      tl[1] = rect.y;
-      tl[2] = 0.;
-      tl[3] = 1.;
+      x1 = rect.x;
+      y1 = rect.y;
+      z1 = 0.f;
+      w1 = 1.f;
 
-      br[0] = rect.x + rect.width;
-      br[1] = rect.y + rect.height;
-      br[2] = 0.;
-      br[3] = 1.;
+      x2 = rect.x + rect.width;
+      y2 = rect.y + rect.height;
+      z2 = 0.f;
+      w2 = 1.f;
 
-      cogl_matrix_transform_point (&matrix, &tl[0], &tl[1], &tl[2], &tl[3]);
-      cogl_matrix_transform_point (&matrix, &br[0], &br[1], &br[2], &br[3]);
+      cogl_matrix_transform_point (&matrix, &x1, &y1, &z1, &w1);
+      cogl_matrix_transform_point (&matrix, &x2, &y2, &z2, &w2);
 
-      _cogl_rectangle_immediate (framebuffer,
-                                 ctx->stencil_pipeline,
-                                 tl[0], tl[1], br[0], br[1]);
+      v[0].x = x1;
+      v[0].y = y1;
+      v[1].x = x1;
+      v[1].y = y2;
+      v[2].x = x2;
+      v[2].y = y1;
+      v[3].x = x1;
+      v[3].y = y2;
+      v[4].x = x2;
+      v[4].y = y2;
+      v[5].x = x2;
+      v[5].y = y1;
     }
 
+  cogl_2d_primitives_immediate (framebuffer,
+                                ctx->stencil_pipeline,
+                                COGL_VERTICES_MODE_TRIANGLES,
+                                vertices,
+                                6 * num_rectangles);
+
   if (merge)
     {
       /* Subtract one from all pixels in the stencil buffer so that
