@@ -282,13 +282,15 @@ pick_keycode_for_keyval_in_current_group (ClutterVirtualInputDevice *virtual_dev
 {
   MetaVirtualInputDeviceNative *virtual_evdev =
     META_VIRTUAL_INPUT_DEVICE_NATIVE (virtual_device);
+  ClutterBackend *backend;
   ClutterKeymap *keymap;
   struct xkb_keymap *xkb_keymap;
   struct xkb_state  *state;
   guint keycode, layout;
   xkb_keycode_t min_keycode, max_keycode;
 
-  keymap = clutter_backend_get_keymap (clutter_get_default_backend ());
+  backend = clutter_get_default_backend ();
+  keymap = clutter_seat_get_keymap (clutter_backend_get_default_seat (backend));
   xkb_keymap = meta_keymap_native_get_keyboard_map (META_KEYMAP_NATIVE (keymap));
   state = virtual_evdev->seat->xkb;
 
@@ -630,21 +632,22 @@ meta_virtual_input_device_native_constructed (GObject *object)
     CLUTTER_VIRTUAL_INPUT_DEVICE (object);
   MetaVirtualInputDeviceNative *virtual_evdev =
     META_VIRTUAL_INPUT_DEVICE_NATIVE (object);
-  ClutterDeviceManager *manager;
   ClutterInputDeviceType device_type;
   ClutterStage *stage;
 
-  manager = clutter_virtual_input_device_get_manager (virtual_device);
   device_type = clutter_virtual_input_device_get_device_type (virtual_device);
 
   virtual_evdev->device =
-    meta_input_device_native_new_virtual (manager,
-                                          virtual_evdev->seat,
+    meta_input_device_native_new_virtual (virtual_evdev->seat,
                                           device_type,
                                           CLUTTER_INPUT_MODE_SLAVE);
 
-  stage = meta_device_manager_native_get_stage (META_DEVICE_MANAGER_NATIVE (manager));
+  stage = meta_seat_native_get_stage (virtual_evdev->seat);
   _clutter_input_device_set_stage (virtual_evdev->device, stage);
+
+  g_signal_emit_by_name (virtual_evdev->seat,
+                         "device-added",
+                         virtual_evdev->device);
 }
 
 static void
@@ -657,6 +660,10 @@ meta_virtual_input_device_native_finalize (GObject *object)
   GObjectClass *object_class;
 
   release_pressed_buttons (virtual_device);
+  g_signal_emit_by_name (virtual_evdev->seat,
+                         "device-removed",
+                         virtual_evdev->device);
+
   g_clear_object (&virtual_evdev->device);
 
   object_class =
