@@ -251,6 +251,21 @@ meta_window_actor_is_frozen (MetaWindowActor *self)
   return priv->surface == NULL || priv->freeze_count > 0;
 }
 
+static void
+meta_window_actor_set_frozen (MetaWindowActor *self,
+                              gboolean         frozen)
+{
+  MetaWindowActorPrivate *priv =
+    meta_window_actor_get_instance_private (self);
+
+  if (meta_surface_actor_is_frozen (priv->surface) == frozen)
+    return;
+
+  meta_surface_actor_set_frozen (priv->surface, frozen);
+
+  META_WINDOW_ACTOR_GET_CLASS (self)->set_frozen (self, frozen);
+}
+
 static void
 meta_window_actor_freeze (MetaWindowActor *self)
 {
@@ -258,7 +273,7 @@ meta_window_actor_freeze (MetaWindowActor *self)
     meta_window_actor_get_instance_private (self);
 
   if (priv->freeze_count == 0 && priv->surface)
-    meta_surface_actor_set_frozen (priv->surface, TRUE);
+    meta_window_actor_set_frozen (self, TRUE);
 
   priv->freeze_count ++;
 }
@@ -273,7 +288,7 @@ meta_window_actor_sync_thawed_state (MetaWindowActor *self)
     priv->first_frame_state = DRAWING_FIRST_FRAME;
 
   if (priv->surface)
-    meta_surface_actor_set_frozen (priv->surface, FALSE);
+    meta_window_actor_set_frozen (self, FALSE);
 
   /* We sometimes ignore moves and resizes on frozen windows */
   meta_window_actor_sync_actor_geometry (self, FALSE);
@@ -312,7 +327,7 @@ meta_window_actor_real_assign_surface_actor (MetaWindowActor  *self,
   priv->surface = g_object_ref_sink (surface_actor);
 
   if (meta_window_actor_is_frozen (self))
-    meta_surface_actor_set_frozen (priv->surface, TRUE);
+    meta_window_actor_set_frozen (self, TRUE);
   else
     meta_window_actor_sync_thawed_state (self);
 }
@@ -957,20 +972,28 @@ meta_window_actor_cull_out (MetaCullable   *cullable,
 
   meta_cullable_cull_out_children (cullable, unobscured_region, clip_region);
 
-  if (unobscured_region && meta_window_actor_is_opaque (self))
+  if ((unobscured_region || clip_region) && meta_window_actor_is_opaque (self))
     {
       cairo_region_t *region = meta_window_get_frame_bounds (priv->window);
 
       if (region)
         {
-          cairo_region_subtract (unobscured_region, region);
+          if (unobscured_region)
+            cairo_region_subtract (unobscured_region, region);
+          if (clip_region)
+            cairo_region_subtract (clip_region, region);
         }
       else
         {
           cairo_rectangle_int_t rect;
+
           meta_window_get_frame_rect (priv->window, &rect);
           rect.x = rect.y = 0;
-          cairo_region_subtract_rectangle (unobscured_region, &rect);
+
+          if (unobscured_region)
+            cairo_region_subtract_rectangle (unobscured_region, &rect);
+          if (clip_region)
+            cairo_region_subtract_rectangle (clip_region, &rect);
         }
     }
 }
