@@ -264,7 +264,7 @@ clutter_test_run (void)
 typedef struct {
   ClutterActor *stage;
 
-  ClutterPoint point;
+  graphene_point_t point;
 
   gpointer result;
 
@@ -339,13 +339,13 @@ on_key_press_event (ClutterActor *stage,
  * Since: 1.18
  */
 gboolean
-clutter_test_check_actor_at_point (ClutterActor        *stage,
-                                   const ClutterPoint  *point,
-                                   ClutterActor        *actor,
-                                   ClutterActor       **result)
+clutter_test_check_actor_at_point (ClutterActor            *stage,
+                                   const graphene_point_t  *point,
+                                   ClutterActor            *actor,
+                                   ClutterActor           **result)
 {
   ValidateData *data;
-  guint press_id = 0;
+  gulong press_id = 0;
 
   g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
   g_return_val_if_fail (point != NULL, FALSE);
@@ -377,8 +377,7 @@ clutter_test_check_actor_at_point (ClutterActor        *stage,
 
   *result = data->result;
 
-  if (press_id != 0)
-    g_signal_handler_disconnect (stage, press_id);
+  g_clear_signal_handler (&press_id, stage);
 
   g_free (data);
 
@@ -401,15 +400,15 @@ clutter_test_check_actor_at_point (ClutterActor        *stage,
  * Since: 1.18
  */
 gboolean
-clutter_test_check_color_at_point (ClutterActor       *stage,
-                                   const ClutterPoint *point,
-                                   const ClutterColor *color,
-                                   ClutterColor       *result)
+clutter_test_check_color_at_point (ClutterActor           *stage,
+                                   const graphene_point_t *point,
+                                   const ClutterColor     *color,
+                                   ClutterColor           *result)
 {
   ValidateData *data;
   gboolean retval;
   guint8 *buffer;
-  guint press_id = 0;
+  gulong press_id = 0;
 
   g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
   g_return_val_if_fail (point != NULL, FALSE);
@@ -439,8 +438,7 @@ clutter_test_check_color_at_point (ClutterActor       *stage,
   while (!data->was_painted)
     g_main_context_iteration (NULL, TRUE);
 
-  if (press_id != 0)
-    g_signal_handler_disconnect (stage, press_id);
+  g_clear_signal_handler (&press_id, stage);
 
   buffer = data->result;
 
