@@ -206,7 +206,7 @@ meta_wayland_surface_assign_role (MetaWaylandSurface *surface,
                                           (const GValue *) values->data);
           surface->role = META_WAYLAND_SURFACE_ROLE (role_object);
 
-          g_array_free (names, FALSE);
+          g_array_free (names, TRUE);
           g_array_free (values, TRUE);
         }
       else
@@ -600,20 +600,6 @@ meta_wayland_surface_state_class_init (MetaWaylandSurfaceStateClass *klass)
                   G_TYPE_NONE, 0);
 }
 
-static void
-parent_surface_state_applied (GNode    *subsurface_node,
-                              gpointer  user_data)
-{
-  MetaWaylandSurface *surface = subsurface_node->data;
-  MetaWaylandSubsurface *subsurface;
-
-  if (G_NODE_IS_LEAF (subsurface_node))
-    return;
-
-  subsurface = META_WAYLAND_SUBSURFACE (surface->role);
-  meta_wayland_subsurface_parent_state_applied (subsurface);
-}
-
 void
 meta_wayland_surface_cache_pending_frame_callbacks (MetaWaylandSurface      *surface,
                                                     MetaWaylandSurfaceState *pending)
@@ -627,6 +613,7 @@ static void
 meta_wayland_surface_apply_state (MetaWaylandSurface      *surface,
                                   MetaWaylandSurfaceState *state)
 {
+  MetaWaylandSurface *subsurface_surface;
   gboolean had_damage = FALSE;
 
   g_signal_emit (surface, surface_signals[SURFACE_PRE_STATE_APPLIED], 0);
@@ -665,11 +652,9 @@ meta_wayland_surface_apply_state (MetaWaylandSurface      *surface,
       if (state->buffer)
         {
           GError *error = NULL;
-          gboolean changed_texture;
 
           if (!meta_wayland_buffer_attach (state->buffer,
                                            &surface->texture,
-                                           &changed_texture,
                                            &error))
             {
               g_warning ("Could not import pending buffer: %s", error->message);
@@ -680,24 +665,6 @@ meta_wayland_surface_apply_state (MetaWaylandSurface      *surface,
               g_error_free (error);
               goto cleanup;
             }
-
-          if (changed_texture && meta_wayland_surface_get_actor (surface))
-            {
-              MetaShapedTexture *stex;
-              CoglTexture *texture;
-              CoglSnippet *snippet;
-              gboolean is_y_inverted;
-
-              stex = meta_surface_actor_get_texture (meta_wayland_surface_get_actor (surface));
-              texture = surface->texture;
-              snippet = meta_wayland_buffer_create_snippet (state->buffer);
-              is_y_inverted = meta_wayland_buffer_is_y_inverted (state->buffer);
-
-              meta_shaped_texture_set_texture (stex, texture);
-              meta_shaped_texture_set_snippet (stex, snippet);
-              meta_shaped_texture_set_is_y_inverted (stex, is_y_inverted);
-              g_clear_pointer (&snippet, cogl_object_unref);
-            }
         }
       else
         {
@@ -802,10 +769,13 @@ cleanup:
 
   meta_wayland_surface_state_reset (state);
 
-  g_node_children_foreach (surface->subsurface_branch_node,
-                           G_TRAVERSE_ALL,
-                           parent_surface_state_applied,
-                           NULL);
+  META_WAYLAND_SURFACE_FOREACH_SUBSURFACE (surface, subsurface_surface)
+    {
+      MetaWaylandSubsurface *subsurface;
+
+      subsurface = META_WAYLAND_SUBSURFACE (subsurface_surface->role);
+      meta_wayland_subsurface_parent_state_applied (subsurface);
+    }
 
   if (had_damage)
     {
@@ -1287,22 +1257,14 @@ meta_wayland_surface_update_outputs (MetaWaylandSurface *surface)
 static void
 meta_wayland_surface_update_outputs_recursively (MetaWaylandSurface *surface)
 {
-  GNode *n;
+  MetaWaylandSurface *subsurface_surface;
 
   meta_wayland_surface_update_outputs (surface);
 
-  for (n = g_node_first_child (surface->subsurface_branch_node);
-       n;
-       n = g_node_next_sibling (n))
-    {
-      if (G_NODE_IS_LEAF (n))
-        continue;
-
-      meta_wayland_surface_update_outputs_recursively (n->data);
-    }
+  META_WAYLAND_SURFACE_FOREACH_SUBSURFACE (surface, subsurface_surface)
+    meta_wayland_surface_update_outputs_recursively (subsurface_surface);
 }
 
-
 void
 meta_wayland_surface_set_window (MetaWaylandSurface *surface,
                                  MetaWindow         *window)
@@ -1428,6 +1390,7 @@ window_actor_effects_completed (MetaWindowActor    *window_actor,
                                 MetaWaylandSurface *surface)
 {
   meta_wayland_surface_update_outputs_recursively (surface);
+  meta_wayland_compositor_repick (surface->compositor);
 }
 
 MetaWaylandSurface *
