@@ -26,31 +26,23 @@
 #define META_WINDOW_WAYLAND_H
 
 #include "core/window-private.h"
-#include <meta/window.h>
+#include "meta/window.h"
 #include "wayland/meta-wayland-types.h"
 
 G_BEGIN_DECLS
 
-#define META_TYPE_WINDOW_WAYLAND            (meta_window_wayland_get_type())
-#define META_WINDOW_WAYLAND(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), META_TYPE_WINDOW_WAYLAND, MetaWindowWayland))
-#define META_WINDOW_WAYLAND_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  META_TYPE_WINDOW_WAYLAND, MetaWindowWaylandClass))
-#define META_IS_WINDOW_WAYLAND(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), META_TYPE_WINDOW_WAYLAND))
-#define META_IS_WINDOW_WAYLAND_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  META_TYPE_WINDOW_WAYLAND))
-#define META_WINDOW_WAYLAND_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  META_TYPE_WINDOW_WAYLAND, MetaWindowWaylandClass))
-
-GType meta_window_wayland_get_type (void);
-
-typedef struct _MetaWindowWayland      MetaWindowWayland;
-typedef struct _MetaWindowWaylandClass MetaWindowWaylandClass;
+#define META_TYPE_WINDOW_WAYLAND (meta_window_wayland_get_type())
+G_DECLARE_FINAL_TYPE (MetaWindowWayland, meta_window_wayland,
+                      META, WINDOW_WAYLAND,
+                      MetaWindow)
 
 MetaWindow * meta_window_wayland_new       (MetaDisplay        *display,
                                             MetaWaylandSurface *surface);
 
-void meta_window_wayland_move_resize (MetaWindow        *window,
-                                      MetaWaylandSerial *acked_configure_serial,
-                                      MetaRectangle      new_geom,
-                                      int                dx,
-                                      int                dy);
+void meta_window_wayland_finish_move_resize (MetaWindow              *window,
+                                             MetaRectangle            new_geom,
+                                             MetaWaylandSurfaceState *pending);
+
 int meta_window_wayland_get_geometry_scale (MetaWindow *window);
 
 void meta_window_wayland_place_relative_to (MetaWindow *window,
@@ -78,5 +70,4 @@ void meta_window_wayland_get_max_size (MetaWindow *window,
                                        int        *width,
                                        int        *height);
 
-
 #endif
