@@ -23,10 +23,11 @@
 #ifndef META_WINDOW_X11_H
 #define META_WINDOW_X11_H
 
-#include <meta/window.h>
-#include <meta/compositor.h>
 #include <X11/Xlib.h>
 
+#include "meta/compositor.h"
+#include "meta/window.h"
+
 G_BEGIN_DECLS
 
 #define META_TYPE_WINDOW_X11            (meta_window_x11_get_type())
@@ -41,6 +42,8 @@ GType meta_window_x11_get_type (void);
 typedef struct _MetaWindowX11      MetaWindowX11;
 typedef struct _MetaWindowX11Class MetaWindowX11Class;
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaWindowX11, g_object_unref)
+
 MetaWindow * meta_window_x11_new           (MetaDisplay        *display,
                                             Window              xwindow,
                                             gboolean            must_be_viewable,
@@ -48,6 +51,12 @@ MetaWindow * meta_window_x11_new           (MetaDisplay        *display,
 
 void meta_window_x11_set_net_wm_state            (MetaWindow *window);
 void meta_window_x11_set_wm_state                (MetaWindow *window);
+void meta_window_x11_set_wm_take_focus           (MetaWindow *window,
+                                                  gboolean    take_focus);
+void meta_window_x11_set_wm_ping                 (MetaWindow *window,
+                                                  gboolean    ping);
+void meta_window_x11_set_wm_delete_window        (MetaWindow *window,
+                                                  gboolean    delete_window);
 void meta_window_x11_set_allowed_actions_hint    (MetaWindow *window);
 
 void meta_window_x11_create_sync_request_alarm   (MetaWindow *window);
@@ -72,4 +81,16 @@ void     meta_window_x11_configure_notify        (MetaWindow      *window,
 
 Window   meta_window_x11_get_toplevel_xwindow    (MetaWindow *window);
 
+void     meta_window_x11_freeze_commits          (MetaWindow *window);
+void     meta_window_x11_thaw_commits            (MetaWindow *window);
+
+void     meta_window_x11_set_thaw_after_paint    (MetaWindow *window,
+                                                  gboolean    thaw_after_paint);
+gboolean meta_window_x11_should_thaw_after_paint (MetaWindow *window);
+gboolean meta_window_x11_always_update_shape     (MetaWindow *window);
+
+void meta_window_x11_buffer_rect_to_frame_rect   (MetaWindow    *window,
+                                                  MetaRectangle *buffer_rect,
+                                                  MetaRectangle *frame_rect);
+
 #endif
